library lwws;

uses Windows, NUtil, NHttp, NkMonAPI; 

{$E nmp}

type
  TCallback = class(TObject)
    procedure Callback(Sender: TObject);
  end;

var
  FDay: string;
  Url: string;
  City: string;
  AmedasUrl: string;
  Http: THttp;
  CB: TCallback;

function UTF8toSJIS(const S: string): string;
var
  w: string;
  count: Integer;
begin
  SetString(w, nil, Length(S)*2);
  count := MultiByteToWideChar(CP_UTF8, 0, PChar(S), Length(S), PWideChar(w),
    Length(w) div 2); 
  SetLength(w, count*2);
  SetString(Result, nil, count*2);
  SetLength(Result, WideCharToMultiByte(CP_ACP, 0, PWideChar(w), count,
    PChar(Result), count*2, nil, nil));
end;

procedure SetValue(const Name: string; Value: string);
begin
  if Value = '' then
    Value := '--';
  nkmSetValue(HInstance, PChar(FDay+'.'+Name), PChar(Value));
end;

function GetAttr(const S, Name: string): string;
var
  i: Integer;
  q: Boolean;
  attr, token: string;
begin
  Result := '';
  i := 0;
  q := False;
  while i < Length(S) do
  begin
    Inc(i);
    case S[i] of
      '"': q := not q;
      ' ':
        if q then
          token := token + S[i]
        else if attr = Name then
        begin
          Result := token;
          Exit;
        end else
          token := '';
      '=':
        begin
          attr := token;
          token := '';
        end;
    else
       token := token + S[i];
    end;
  end;
end;

procedure Parse(const S: string);
var
  i, j: Integer;
  intag, skip: Boolean;
  tag, attr, cont, name: string;
  tags: TStrList;
begin
  i := 0;
  intag := False;
  skip := False;
  tags := TStrList.Create;
  while i < Length(S) do
  begin
    Inc(i);
    case S[i] of
      '<':
        begin
          intag := True;
          skip := False;
        end;
      '>':
        begin
          intag := False;
          if (tag = '?xml') or ((i > 1) and (S[i-1] = '/')) then
          begin
            if tag = 'location' then
              SetValue('city', Utf8ToSjis(GetAttr(attr, 'city')));
            attr := '';
            tag := '';
            continue;
          end;
          if tag[1] = '/' then
          begin
            name := tags[0];
            for j := 1 to tags.Count-1 do
              name := name + '.' + tags[j];
            cont := Utf8ToSjis(cont);
            if name = 'lwws.day' then
              SetValue('day', Copy(cont, 1, 3))
            else if name = 'lwws.publictime' then
            begin
              SetValue('publictime.day', Copy(cont, 6, 2));
              SetValue('publictime.hour', Copy(cont, 18, 2));
              SetValue('publictime.min', Copy(cont, 21, 2));
            end
            else if name = 'lwws.telop' then
              SetValue('telop', cont)
            else if name = 'lwws.temperature.max.celsius' then
              SetValue('maxtemp', cont)
            else if name = 'lwws.temperature.min.celsius' then
              SetValue('mintemp', cont);
            cont := '';
            tags.Delete(tags.Count-1);
          end else
            tags.Add(tag);
          tag := '';
        end;
      #10: ; // pass 
    else
      if intag then
      begin
        if S[i] = ' ' then
          skip := True;
        if skip then
          attr := attr + S[i]
        else
          tag := tag + S[i]
      end else
        cont := cont + S[i];
    end;
  end;
  tags.Free;
end;

procedure ParseAmedasTitle(const S: string; var Loc, Value0, Value1: string);
var
  i: Integer;
  tmp: string;
begin
  i := 0;
  Loc := '';
  Value0 := '';
  Value1 := '';
  while i < Length(S) do
  begin
    Inc(i);
    case S[i] of
      ' ':
        begin
          if Loc = '' then
            Loc := tmp
          else
            Value0 := tmp;
          tmp := '';
        end;
      ']':
        tmp := '';
    else
      tmp := tmp + S[i];
    end;
  end;
  if Value0 = '' then
    Value0 := tmp
  else
    Value1 := tmp
end;

procedure ParseAmedas(const S: string);
var
  i, j: Integer;
  intag, skip: Boolean;
  tag, attr, cont, name, vname, title, pubdate, loc, value0, value1: string;
  tags: TStrList;
begin
  i := 0;
  intag := False;
  skip := False;
  tags := TStrList.Create;
  while i < Length(S) do
  begin
    Inc(i);
    case S[i] of
      '<':
        begin
          intag := True;
          skip := False;
        end;
      '>':
        begin
          intag := False;
          if (tag = '?xml') or ((i > 1) and (S[i-1] = '/')) then
          begin
            attr := '';
            tag := '';
            continue;
          end;
          if tag[1] = '/' then
          begin
            name := tags[0];
            for j := 1 to tags.Count-1 do
              name := name + '.' + tags[j];
            cont := Utf8ToSjis(cont);
            if name = 'rss.channel.item.title' then
              title := cont
            else if name = 'rss.channel.item.pubDate' then
              pubdate := cont
            else if name = 'rss.channel.item.category' then
            begin
              if cont = 'A_X C' then
                vname := 'temp'
              else if cont = 'A_X ~' then
                vname := 'rain'
              else if cont = 'A_X E' then
                vname := 'wind'
              else if cont = 'A_X Ǝ' then
                vname := 'sunshine'
              else if cont = 'A_X ϐ[' then
                vname := 'snow'
              else
                vname := '';
            end
            else if name = 'rss.channel.item' then
              if vname <> '' then
              begin
                ParseAmedasTitle(title, loc, value0, value1);
                SetValue(vname+'.location', loc);
                if vname = 'wind' then
                begin
                  // É̂Ƃ "É0m/s" ƕƂ̊ԂɃXy[XȂ
                  if copy(value1, 1, 4) = 'É' then
                  begin
                    SetValue(vname+'.direction',
                      Copy(value1, 1, 4));
                    SetValue(vname+'.speed',
                      Copy(value1, 5, Length(value1)-4-3));
                  end
                  else begin 
                    SetValue(vname+'.direction',
                      Copy(value0, 1, Length(value0)-4));
                    SetValue(vname+'.speed',
                      Copy(value1, 1, Length(value1)-3));
                  end;
                end else
                  SetValue(vname, Copy(value0, 1, Length(value0)-2));
                SetValue(vname+'.publictime.day', Copy(pubdate, 6, 2));
                SetValue(vname+'.publictime.hour', Copy(pubdate, 18, 2));
                SetValue(vname+'.publictime.min', Copy(pubdate, 21, 2));
              end;
            cont := '';
            tags.Delete(tags.Count-1);
          end else
            tags.Add(tag);
          tag := '';
        end;
      #10: ; // pass 
    else
      if intag then
      begin
        if S[i] = ' ' then
          skip := True;
        if skip then
          attr := attr + S[i]
        else
          tag := tag + S[i]
      end else
        cont := cont + S[i];
    end;
  end;
  tags.Free;
end;

procedure RestRequest;
begin
  Http.Get(Url + '?city='+City+'&day='+FDay, '');
end;

procedure TCallback.Callback(Sender: TObject);
begin
  with Sender as THttp do
  begin
    if Error <> '' then
      nkmLog(HInstance, PChar(Error));
    if Info <> '' then
      nkmLog(HInstance, PChar(Info));

    if Response.Received then
    begin
      if Response.Status = 200 then
        if FDay = 'amedas' then
          ParseAmedas(Response.Body)
        else
          Parse(Response.Body);

      if FDay = 'today' then
      begin
        FDay := 'tomorrow';
        RestRequest;
      end
      else if FDay = 'tomorrow' then
      begin
        FDay := 'dayaftertomorrow';
        RestRequest;
      end
      else if FDay = 'dayaftertomorrow' then
      begin
        FDay := 'amedas';
        Get(AmedasUrl, '');
      end;
    end;
  end;
end;

function gauge_init: Integer; stdcall;
begin
  Url := nkmConf(HInstance, 'url',
    'http://weather.livedoor.com/forecast/webservice/rest/v1');
  // default is Tokyo
  City := nkmConf(HInstance, 'city', '63');
  // default is Tokyo
  AmedasUrl := nkmConf(HInstance,'amedasurl',
    'http://weather.livedoor.com/forecast/rss/amedas/point/44132.xml');
  CB := TCallback.Create;
  Http := THttp.Create;
  Http.Callback := CB.Callback;
  Result := 3600;
end;

procedure gauge_update; stdcall;
begin
  FDay := 'today';
  RestRequest;
end;

procedure gauge_unload; stdcall;
begin
  Http.Free;
  CB.Free;
end;

exports
  gauge_init,
  gauge_update,
  gauge_unload;

end.
